/*
 * SPDX-FileCopyrightText: 2007 Jeremy Whiting <jpwhiting@kde.org>
 * SPDX-FileCopyrightText: 2007 Frederik Gladhorn <frederik.gladhorn@kdemail.net>
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef KEDUVOCCOMMON_P_H
#define KEDUVOCCOMMON_P_H

#define KVD_VERS_PREFIX " v" // kvoctrain v0.1.0

#endif
